@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2012,2013,2017                           Inria
- * Copyright (C) 2010-2015,2017,2019                      CNRS
- * Copyright (C) 2009-2011,2013-2015                      Université de Bordeaux
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -53,8 +51,6 @@ int main(int argc, char **argv)
 
 	struct starpu_conf conf;
 	starpu_conf_init(&conf);
-	conf.nmic = 0;
-	conf.nmpi_ms = 0;
 
 	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return 77;
