@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2015-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2015-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -83,11 +83,18 @@ void fmultiple_check_scale(void *buffers[], void *cl_arg)
 #ifdef STARPU_USE_CUDA
 extern void fmultiple_check_scale_cuda(void *buffers[], void *cl_arg);
 #endif
+#ifdef STARPU_USE_HIP
+extern void fmultiple_check_scale_hip(void *buffers[], void *cl_arg);
+#endif
 struct starpu_codelet cl_check_scale =
 {
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {fmultiple_check_scale_cuda},
 	.cuda_flags = {STARPU_CUDA_ASYNC},
+#endif
+#ifdef STARPU_USE_HIP
+	.hip_funcs = {fmultiple_check_scale_hip},
+	.hip_flags = {STARPU_HIP_ASYNC},
 #endif
 	.cpu_funcs = {fmultiple_check_scale},
 	.cpu_funcs_name = {"fmultiple_check_scale"},
@@ -121,6 +128,12 @@ int main(void)
 		cl_check_scale.cpu_funcs[0] = NULL;
 		cl_check_scale.cpu_funcs_name[0] = NULL;
 	}
+	/* Disable codelet on CPUs if we have a HIP device, to force remote execution on the HIp device */
+	if (starpu_hip_worker_get_count())
+	{
+		cl_check_scale.cpu_funcs[0] = NULL;
+		cl_check_scale.cpu_funcs_name[0] = NULL;
+	}
 
 	/* Declare the whole matrix to StarPU */
 	starpu_matrix_data_register(&handle, STARPU_MAIN_RAM, (uintptr_t)matrix, NX, NX, NY, sizeof(matrix[0][0]));
