@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010,2011 University of Bordeaux
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -16,16 +16,17 @@
 
 #include "socl.h"
 
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
 soclEnqueueMarker(cl_command_queue  cq,
-                cl_event *          event) CL_API_SUFFIX__VERSION_1_0
+		  cl_event *          event)
 {
 	if (event == NULL)
 		return CL_INVALID_VALUE;
-	
+
 	command_marker cmd = command_marker_create();
 
-   cl_event ev = command_event_get(cmd);
+	cl_event ev = command_event_get(cmd);
 
 	command_queue_enqueue(cq, cmd, 0, NULL);
 
@@ -34,7 +35,8 @@ soclEnqueueMarker(cl_command_queue  cq,
 	return CL_SUCCESS;
 }
 
-cl_int command_marker_submit(command_marker cmd) {
+cl_int command_marker_submit(command_marker cmd)
+{
 	struct starpu_task *task;
 	task = task_create(CL_COMMAND_MARKER);
 
