@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2020       Federal University of Rio Grande do Sul (UFRGS)
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2020-2020  Federal University of Rio Grande do Sul (UFRGS)
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -533,7 +533,7 @@ void _starpu_gethostname(char *hostname, size_t size)
 		int rank;
 
 #ifdef STARPU_HAVE_DARWIN
-		_STARPU_DISP("Detection of MPI rank disabled on darwin, assuming rank is 0");
+		_STARPU_DISP("Detection of MPI rank disabled on darwin, assuming rank is 0\n");
 		rank = 0;
 #else
 		if (starpu_mpi_world_rank)
@@ -542,7 +542,7 @@ void _starpu_gethostname(char *hostname, size_t size)
 		}
 		else
 		{
-			_STARPU_DISP("StarPU-MPI unavailable, the rank of this process is 0");
+			_STARPU_DISP("StarPU-MPI unavailable, the rank of this process is 0\n");
 			rank = 0;
 		}
 #endif
@@ -750,10 +750,15 @@ int starpu_get_env_size_default(const char *str, int defval)
 void starpu_display_bindings(void)
 {
 #if defined(STARPU_HAVE_HWLOC) && !defined(STARPU_SIMGRID)
-	int hwloc_ret = system("hwloc-ps -a -t -c");
-	if (hwloc_ret)
+	int value = starpu_getenv_number_default("STARPU_DISPLAY_BINDINGS", 0);
+	int ret = 0;
+	if (value == 2)
+		ret = system("lstopo --ps -");
+	else
+		ret = system("hwloc-ps -a -t -c");
+	if (ret)
 	{
-		_STARPU_DISP("hwloc-ps returned %d\n", hwloc_ret);
+		_STARPU_DISP("%s returned %d\n", value==2?"lstopo":"hwloc-ps", ret);
 		fflush(stderr);
 	}
 	fflush(stdout);
