@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2024  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Thibaut Lambert
- * Copyright (C) 2016       Uppsala University
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2016-2016  Uppsala University
+ * Copyright (C) 2013-2013  Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +18,7 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <strings.h>
 #include <common/config.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -614,9 +615,13 @@ static void _starpu_init_topology(struct _starpu_machine_config *config)
 
 		hwloc_bitmap_t cpuset = hwloc_bitmap_alloc();
 		hwloc_bitmap_t log_cpuset = hwloc_bitmap_alloc();
+		hwloc_bitmap_t check_cpuset = hwloc_bitmap_alloc();
 		hwloc_bitmap_t log_coreset = hwloc_bitmap_alloc();
-		unsigned n, i, first, last, weight;
+		unsigned n, i, j, first, last, weight;
 		int ret;
+#ifdef STARPU_VERBOSE
+		char *str;
+#endif
 
 		do {
 			/* Get the process binding (e.g. provided by the job scheduler) */
@@ -627,6 +632,12 @@ static void _starpu_init_topology(struct _starpu_machine_config *config)
 				break;
 			}
 
+#ifdef STARPU_VERBOSE
+			hwloc_bitmap_asprintf(&str, cpuset);
+			_STARPU_DEBUG("Got cpu physical binding: %s\n", str);
+			free(str);
+#endif
+
 			/* Compute logical sets */
 			n = hwloc_get_nbobjs_by_depth(topology->hwtopology, config->pu_depth);
 			for (i = 0; i < n; i++)
@@ -649,26 +660,52 @@ static void _starpu_init_topology(struct _starpu_machine_config *config)
 						break;
 					}
 				}
+
+				/* Include all PUs from the core to make the set contiguous, we will pick up just one from it by default */
+				for (j = 0; j < core->arity; j++)
+					hwloc_bitmap_set(check_cpuset, core->children[j]->logical_index);
+
 				hwloc_bitmap_set(log_coreset, core->logical_index);
 			}
 
+#ifdef STARPU_VERBOSE
+			hwloc_bitmap_asprintf(&str, log_cpuset);
+			_STARPU_DEBUG("This maps to logical binding: %s\n", str);
+			free(str);
+			hwloc_bitmap_asprintf(&str, check_cpuset);
+			_STARPU_DEBUG("Which we extend to: %s\n", str);
+			free(str);
+			hwloc_bitmap_asprintf(&str, log_coreset);
+			_STARPU_DEBUG("The logical core binding is thus: %s\n", str);
+			free(str);
+#endif
+
 			/* Check that PU numbers are consecutive */
-			first = hwloc_bitmap_first(log_cpuset);
-			last = hwloc_bitmap_last(log_cpuset);
-			weight = hwloc_bitmap_weight(log_cpuset);
+			first = hwloc_bitmap_first(check_cpuset);
+			last = hwloc_bitmap_last(check_cpuset);
+			weight = hwloc_bitmap_weight(check_cpuset);
 			if (last - first + 1 != weight)
 			{
-				_STARPU_DISP("Warning: hwloc reported non-consecutive binding, this is not supported yet, sorry, please use STARPU_WORKERS_CPUID or STARPU_WORKERS_COREID to set this by hand\n");
+				_STARPU_DISP("Warning: hwloc reported non-consecutive binding (first %u last %d weight %u, this is not supported yet, sorry, please use STARPU_WORKERS_CPUID or STARPU_WORKERS_COREID to set this by hand\n", first, last, weight);
 				break;
 			}
 
-			if (weight == 1 || hwloc_bitmap_weight(log_coreset) == 1)
-				_STARPU_DISP("Warning: the current CPU binding set contains only one CPU, maybe you need to tell your job scheduler to bind on all allocated cores (e.g. --exclusive --ntasks-per-node=1 or --cpus-per-task for Slurm, or --bind-to board for openmpi), or you can use STARPU_WORKERS_GETBIND=0 to bypass it, but make sure you are not oversubscribing the machine\n");
+			if (hwloc_bitmap_weight(log_cpuset) == 1 || hwloc_bitmap_weight(log_coreset) == 1)
+			{
+				const char *omp_bind = starpu_getenv("OMP_PROC_BIND");
+				_STARPU_DISP("Warning: the current CPU binding set contains only one CPU.\n");
+				if (omp_bind && strcasecmp(omp_bind, "false"))
+					_STARPU_DISP("The OMP_PROC_BIND environment variable is set to %s.\n", omp_bind);
+				else
+					_STARPU_DISP("Maybe you need to tell your job scheduler to bind on all allocated cores (e.g. --exclusive --ntasks-per-node=1 or --cpus-per-task for Slurm, or --bind-to board for openmpi).\n");
+				_STARPU_DISP("You can use STARPU_WORKERS_GETBIND=0 to bypass it, but make sure you are not oversubscribing the machine.\n");
+			}
 			topology->nusedpus = weight;
-			topology->firstusedpu = first;
+			topology->firstusedpu = hwloc_bitmap_first(log_cpuset);;
 		} while(0);
 
 		hwloc_bitmap_free(cpuset);
+		hwloc_bitmap_free(check_cpuset);
 		topology->log_cpuset = log_cpuset;
 		topology->log_coreset = log_coreset;
 	}
@@ -880,7 +917,15 @@ static void _starpu_initialize_workers_bindid(struct _starpu_machine_config *con
 			}
 
 			/* Add a worker to this core, by using this logical PU */
-			topology->workers_bindid[nbindids++] = (unsigned)i + topology->firstusedpu;
+			unsigned allocated = topology->firstusedpu + (unsigned)i;
+
+#if defined(STARPU_HAVE_HWLOC)
+			if (config->topology.log_cpuset &&
+				!hwloc_bitmap_isset(config->topology.log_cpuset, allocated))
+				_STARPU_DISP("Warning: logical CPU id %u is not in the CPU binding provided by the OS, did you specify an STARPU_NTHREADS_PER_CORE value that is not covered by the OS-provided CPU binding?\n", allocated);
+#endif
+
+			topology->workers_bindid[nbindids++] = allocated;
 			k++;
 			i++;
 		}
@@ -1767,7 +1812,7 @@ static void _starpu_init_numa_node(struct _starpu_machine_config *config)
 					int devid = numa_logical_id == STARPU_NUMA_MAIN_RAM ? 0 : numa_logical_id;
 					int memnode = _starpu_memory_node_register(STARPU_CPU_RAM, devid);
 					_starpu_memory_manager_set_global_memory_size(memnode, _starpu_cpu_get_global_mem_size(devid, config));
-					STARPU_ASSERT_MSG(memnode < STARPU_MAXNUMANODES, "Wrong Memory Node : %d (only %d available)", memnode, STARPU_MAXNUMANODES);
+					STARPU_ASSERT_MSG_ALWAYS(memnode < STARPU_MAXNUMANODES, "Wrong Memory Node : %d (only %d available)", memnode, STARPU_MAXNUMANODES);
 					_starpu_memory_node_set_mapped(memnode);
 					numa_memory_nodes_to_hwloclogid[memnode] = numa_logical_id;
 					int numa_physical_id = _starpu_get_physical_numa_node_worker(worker);
@@ -1822,7 +1867,7 @@ static void _starpu_init_numa_node(struct _starpu_machine_config *config)
 				{
 					int memnode = _starpu_memory_node_register(STARPU_CPU_RAM, obj->logical_index);
 					_starpu_memory_manager_set_global_memory_size(memnode, _starpu_cpu_get_global_mem_size(obj->logical_index, config));
-					STARPU_ASSERT_MSG(memnode < STARPU_MAXNUMANODES, "Wrong Memory Node : %d (only %d available)", memnode, STARPU_MAXNUMANODES);
+					STARPU_ASSERT_MSG_ALWAYS(memnode < STARPU_MAXNUMANODES, "Wrong Memory Node : %d (only %d available)", memnode, STARPU_MAXNUMANODES);
 					_starpu_memory_node_set_mapped(memnode);
 					numa_memory_nodes_to_hwloclogid[memnode] = obj->logical_index;
 					numa_memory_nodes_to_physicalid[memnode] = obj->os_index;
@@ -1888,7 +1933,7 @@ static void _starpu_init_numa_node(struct _starpu_machine_config *config)
 
 		if (numa == 0)
 			STARPU_ASSERT_MSG(memnode == STARPU_MAIN_RAM, "Wrong Memory Node : %d (expected %d) \n", memnode, STARPU_MAIN_RAM);
-		STARPU_ASSERT_MSG(memnode < STARPU_MAXNUMANODES, "Wrong Memory Node : %d (only %d available) \n", memnode, STARPU_MAXNUMANODES);
+		STARPU_ASSERT_MSG_ALWAYS(memnode < STARPU_MAXNUMANODES, "Wrong Memory Node : %d (only %d available) \n", memnode, STARPU_MAXNUMANODES);
 
 #ifdef STARPU_SIMGRID
 		if (nnuma > 1)
@@ -2196,8 +2241,10 @@ void starpu_topology_print(FILE *output)
 		fprintf(output, "\t");
 #endif
 		if ((pu % nthreads_per_core) == 0)
-			fprintf(output, "core %u", pu / nthreads_per_core);
-		fprintf(output, "\tPU %u\t", pu);
+			fprintf(output, "core %-5u ", pu / nthreads_per_core);
+		else
+			fprintf(output, "           ");
+		fprintf(output, "PU %-5u ", pu);
 		for (worker = 0;
 		     worker < nworkers + ncombinedworkers;
 		     worker++)
@@ -2210,7 +2257,7 @@ void starpu_topology_print(FILE *output)
 				{
 					char name[256];
 					starpu_worker_get_name(worker, name, sizeof(name));
-					fprintf(output, "%s\t", name);
+					fprintf(output, "%-10s ", name);
 				}
 			}
 			else
@@ -2221,7 +2268,7 @@ void starpu_topology_print(FILE *output)
 				for (i = 0; i < worker_size; i++)
 				{
 					if (topology->workers_bindid[combined_workerid[i]] == pu)
-						fprintf(output, "comb %u\t", worker-nworkers);
+						fprintf(output, "comb %-5u ", worker-nworkers);
 				}
 			}
 		}
