@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2013       Thibaut Lambert
+ * Copyright (C) 2009-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2013  Thibaut Lambert
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -46,7 +46,7 @@ void starpu_data_set_reduction_methods_with_args(starpu_data_handle_t handle, st
 			STARPU_HG_DISABLE_CHECKING(_warned);
 			if (!_warned)
 			{
-				_STARPU_DISP("Warning: The reduction method should use STARPU_COMMUTE for its first parameter");
+				_STARPU_DISP("Warning: The reduction method should use STARPU_COMMUTE for its first parameter\n");
 				_warned = 1;
 			}
 			redux_cl->modes[0] |= STARPU_COMMUTE;
