@@ -1,8 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2011	    Télécom-SudParis
- * Copyright (C) 2013	    Thibaut Lambert
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2013-2013  Thibaut Lambert
+ * Copyright (C) 2011-2011  Télécom Sud Paris
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -118,6 +118,11 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 				worker->cl_expend.tv_nsec = 0;
 			}
 		}
+		else
+		{
+			worker->cl_start.tv_sec = 0;
+			worker->cl_start.tv_nsec = 0;
+		}
 
 		_starpu_job_notify_start(j, perf_arch);
 	}
@@ -149,12 +154,12 @@ void _starpu_driver_start_job(struct _starpu_worker *worker, struct _starpu_job
 			}
 		}
 		_STARPU_TRACE_TASK_NAME_LINE_COLOR(j);
-		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, &sched_ctx->perf_arch, workerid);
+		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, &sched_ctx->perf_arch, workerid, rank);
 	}
 	else
 	{
 		_STARPU_TRACE_TASK_NAME_LINE_COLOR(j);
-		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, perf_arch, workerid);
+		_STARPU_TRACE_START_CODELET_BODY(j, j->nimpl, perf_arch, workerid, rank);
 	}
 	_starpu_sched_ctx_unlock_read(sched_ctx->id);
 	_STARPU_TASK_BREAK_ON(task, exec);
@@ -175,12 +180,12 @@ void _starpu_driver_end_job(struct _starpu_worker *worker, struct _starpu_job *j
 	if (!sched_ctx->sched_policy)
 	{
 		_starpu_perfmodel_create_comb_if_needed(&(sched_ctx->perf_arch));
-		_STARPU_TRACE_END_CODELET_BODY(j, j->nimpl, &(sched_ctx->perf_arch), workerid);
+		_STARPU_TRACE_END_CODELET_BODY(j, j->nimpl, &(sched_ctx->perf_arch), workerid, rank);
 	}
 	else
 	{
 		_starpu_perfmodel_create_comb_if_needed(perf_arch);
-		_STARPU_TRACE_END_CODELET_BODY(j, j->nimpl, perf_arch, workerid);
+		_STARPU_TRACE_END_CODELET_BODY(j, j->nimpl, perf_arch, workerid, rank);
 	}
 
 	if (cl && cl->model && cl->model->benchmarking)
@@ -188,13 +193,14 @@ void _starpu_driver_end_job(struct _starpu_worker *worker, struct _starpu_job *j
 
 	struct timespec end;
 	struct starpu_profiling_task_info *profiling_info = task->profiling_info;
-	if ((profiling && profiling_info) || (rank == 0 && (calibrate_model || !_starpu_perf_counter_paused())))
+	if ((profiling && profiling_info) || (rank == 0 && (calibrate_model || !_starpu_perf_counter_paused() ||
+		                                            worker->cl_start.tv_sec || worker->cl_start.tv_nsec)))
 		_starpu_clock_gettime(&end);
 	_starpu_clear_worker_status(worker, STATUS_INDEX_EXECUTING, &end);
 
 	if (rank == 0)
 	{
-		if ((profiling && profiling_info) || calibrate_model || !_starpu_perf_counter_paused())
+		if (worker->cl_start.tv_sec || worker->cl_start.tv_nsec)
 			worker->cl_end = end;
 		STARPU_AYU_POSTRUNTASK(j->job_id);
 	}
@@ -239,7 +245,7 @@ void _starpu_driver_update_job_feedback(struct _starpu_job *j, struct _starpu_wo
 		calibrate_model = 1;
 #endif
 
-	if ((profiling && profiling_info) || calibrate_model || !_starpu_perf_counter_paused())
+	if (worker->cl_start.tv_sec || worker->cl_start.tv_nsec)
 	{
 		starpu_timespec_sub(&worker->cl_end, &worker->cl_start, &measured_ts);
 		double measured = starpu_timing_timespec_to_us(&measured_ts);
