@@ -1,12 +1,12 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2008-2023  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
- * Copyright (C) 2010	    Mehdi Juhoor
- * Copyright (C) 2011	    Télécom-SudParis
- * Copyright (C) 2013	    Thibaut Lambert
- * Copyright (C) 2016	    Uppsala University
- * Copyright (C) 2021	    Federal University of Rio Grande do Sul (UFRGS)
- * Copyright (C) 2022	    Camille Coti
+ * Copyright (C) 2008-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2022-2022  École de Technologie Supérieure (ETS, Montréal)
+ * Copyright (C) 2021-2021  Federal University of Rio Grande do Sul (UFRGS)
+ * Copyright (C) 2016-2016  Uppsala University
+ * Copyright (C) 2013-2013  Thibaut Lambert
+ * Copyright (C) 2011-2011  Télécom Sud Paris
+ * Copyright (C) 2010-2010  Mehdi Juhoor
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -48,9 +48,6 @@
 /* Consider a rough 10% overhead cost */
 #define FREE_MARGIN 0.9
 
-/* the number of HIP devices */
-static int nhipgpus = -1;
-
 static size_t global_mem[STARPU_MAXHIPDEVS];
 int _starpu_hip_bus_ids[STARPU_MAXHIPDEVS+STARPU_MAXNUMANODES][STARPU_MAXHIPDEVS+STARPU_MAXNUMANODES];
 static hipStream_t streams[STARPU_NMAXWORKERS];
@@ -75,6 +72,7 @@ static starpu_pthread_mutex_t hip_device_init_mutex[STARPU_MAXHIPDEVS];
 static starpu_pthread_cond_t hip_device_init_cond[STARPU_MAXHIPDEVS];
 static int hip_globalbindid;
 
+static int _starpu_hip_peer_access(int devid, int peer_devid);
 
 int _starpu_nworker_per_hip;
 
@@ -145,7 +143,7 @@ void _starpu_hip_init(void)
 
 /* Return the number of devices usable in the system.
  * The value returned cannot be greater than MAXHIPDEVS */
-unsigned _starpu_get_hip_device_count(void)
+static unsigned _starpu_get_hip_device_count(void)
 {
 	int cnt;
 	hipError_t hipres;
@@ -164,11 +162,6 @@ unsigned _starpu_get_hip_device_count(void)
 /* This is run from initialize to determine the number of HIP devices */
 void _starpu_init_hip(void)
 {
-	if (nhipgpus < 0)
-	{
-		nhipgpus = _starpu_get_hip_device_count();
-		STARPU_ASSERT(nhipgpus <= STARPU_MAXHIPDEVS);
-	}
 }
 
 /* This is called to really discover the hardware */
@@ -226,7 +219,7 @@ void _starpu_init_hip_config(struct _starpu_machine_topology *topology, struct _
 	int nworker_per_hip = starpu_get_env_number_default("STARPU_NWORKER_PER_HIP", 1);
 
 	STARPU_ASSERT_MSG(nworker_per_hip > 0, "STARPU_NWORKER_PER_HIP has to be > 0");
-	STARPU_ASSERT_MSG(nworker_per_hip < STARPU_NMAXWORKERS, "STARPU_NWORKER_PER_HIP (%d) cannot be higher than STARPU_NMAXWORKERS (%d)\n", nworker_per_hip, STARPU_NMAXWORKERS);
+	STARPU_ASSERT_MSG_ALWAYS(nworker_per_hip < STARPU_NMAXWORKERS, "STARPU_NWORKER_PER_HIP (%d) cannot be higher than STARPU_NMAXWORKERS (%d)\n", nworker_per_hip, STARPU_NMAXWORKERS);
 
 #ifndef STARPU_NON_BLOCKING_DRIVERS
 	if (nworker_per_hip > 1)
@@ -490,6 +483,8 @@ static void _starpu_hip_limit_gpu_mem_if_needed(unsigned devid)
 /* Really initialize one device */
 static void init_device_context(unsigned devid, unsigned memnode)
 {
+	STARPU_ASSERT(devid < STARPU_MAXHIPDEVS);
+
 	hipError_t hipres;
 	int attempts = 0;
 
@@ -547,12 +542,17 @@ static void init_device_context(unsigned devid, unsigned memnode)
 	if (STARPU_UNLIKELY(hipres))
 		STARPU_HIP_REPORT_ERROR(hipres);
 
-	int i;
-	for (i = 0; i < nhipgpus; i++)
+	int nworkers = starpu_worker_get_count();
+	int workerid;
+	for (workerid = 0; workerid < nworkers; workerid++)
 	{
-		hipres = starpu_hipStreamCreate(&in_peer_transfer_streams[i][devid]);
-		if (STARPU_UNLIKELY(hipres))
-			STARPU_HIP_REPORT_ERROR(hipres);
+		struct _starpu_worker *worker = _starpu_get_worker_struct(workerid);
+		if (worker->arch == STARPU_HIP_WORKER && worker->subworkerid == 0)
+		{
+			hipres = starpu_hipStreamCreate(&in_peer_transfer_streams[worker->devid][devid]);
+			if (STARPU_UNLIKELY(hipres))
+				STARPU_HIP_REPORT_ERROR(hipres);
+		}
 	}
 
 	STARPU_PTHREAD_MUTEX_LOCK(&hip_device_init_mutex[devid]);
@@ -567,15 +567,20 @@ static void init_device_context(unsigned devid, unsigned memnode)
 /* De-initialize one device */
 static void deinit_device_context(unsigned devid STARPU_ATTRIBUTE_UNUSED)
 {
-	int i;
 	starpu_hip_set_device(devid);
 
 	hipStreamDestroy(in_transfer_streams[devid]);
 	hipStreamDestroy(out_transfer_streams[devid]);
 
-	for (i = 0; i < nhipgpus; i++)
+	int nworkers = starpu_worker_get_count();
+	int workerid;
+	for (workerid = 0; workerid < nworkers; workerid++)
 	{
-		hipStreamDestroy(in_peer_transfer_streams[i][devid]);
+		struct _starpu_worker *worker = _starpu_get_worker_struct(workerid);
+		if (worker->arch == STARPU_HIP_WORKER && worker->subworkerid == 0)
+		{
+			hipStreamDestroy(in_peer_transfer_streams[worker->devid][devid]);
+		}
 	}
 }
 
@@ -787,7 +792,7 @@ int _starpu_hip_driver_deinit(struct _starpu_worker *worker)
 	return 0;
 }
 
-uintptr_t _starpu_hip_malloc_on_node(unsigned dst_node, size_t size, int flags)
+static uintptr_t _starpu_hip_malloc_on_node(unsigned dst_node, size_t size, int flags)
 {
 	uintptr_t addr = 0;
 	(void) flags;
@@ -817,7 +822,7 @@ uintptr_t _starpu_hip_malloc_on_node(unsigned dst_node, size_t size, int flags)
 	return addr;
 }
 
-void _starpu_hip_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
+static void _starpu_hip_free_on_node(unsigned dst_node, uintptr_t addr, size_t size, int flags)
 {
 	(void) size;
 	(void) flags;
@@ -954,7 +959,7 @@ static inline hipEvent_t *_starpu_hip_event(union _starpu_async_channel_event *_
 	return event;
 }
 
-unsigned _starpu_hip_test_request_completion(struct _starpu_async_channel *async_channel)
+static unsigned _starpu_hip_test_request_completion(struct _starpu_async_channel *async_channel)
 {
 	hipEvent_t event;
 	hipError_t hipres;
@@ -972,7 +977,8 @@ unsigned _starpu_hip_test_request_completion(struct _starpu_async_channel *async
 	return success;
 }
 
-void _starpu_hip_wait_request_completion(struct _starpu_async_channel *async_channel)
+/* Only used at starpu_shutdown */
+static void _starpu_hip_wait_request_completion(struct _starpu_async_channel *async_channel)
 {
 	hipEvent_t event;
 	hipError_t hipres;
@@ -1008,7 +1014,7 @@ static void starpu_hip_set_copy_device(unsigned src_node, unsigned dst_node)
 }
 #endif
 
-int _starpu_hip_copy_interface_from_hip_to_hip(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+static int _starpu_hip_copy_interface_from_hip_to_hip(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -1047,7 +1053,7 @@ int _starpu_hip_copy_interface_from_hip_to_hip(starpu_data_handle_t handle, void
 	return ret;
 }
 
-int _starpu_hip_copy_interface_from_hip_to_cpu(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+static int _starpu_hip_copy_interface_from_hip_to_cpu(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -1088,7 +1094,7 @@ int _starpu_hip_copy_interface_from_hip_to_cpu(starpu_data_handle_t handle, void
 	return ret;
 }
 
-int _starpu_hip_copy_interface_from_cpu_to_hip(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
+static int _starpu_hip_copy_interface_from_cpu_to_hip(starpu_data_handle_t handle, void *src_interface, unsigned src_node, void *dst_interface, unsigned dst_node, struct _starpu_data_request *req)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -1133,7 +1139,7 @@ int _starpu_hip_copy_interface_from_cpu_to_hip(starpu_data_handle_t handle, void
 	return ret;
 }
 
-int _starpu_hip_copy_data_from_hip_to_cpu(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+static int _starpu_hip_copy_data_from_hip_to_cpu(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -1147,7 +1153,7 @@ int _starpu_hip_copy_data_from_hip_to_cpu(uintptr_t src, size_t src_offset, unsi
 					  hipMemcpyDeviceToHost);
 }
 
-int _starpu_hip_copy_data_from_hip_to_hip(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+static int _starpu_hip_copy_data_from_hip_to_hip(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -1164,7 +1170,7 @@ int _starpu_hip_copy_data_from_hip_to_hip(uintptr_t src, size_t src_offset, unsi
 					  hipMemcpyDeviceToDevice);
 }
 
-int _starpu_hip_copy_data_from_cpu_to_hip(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
+static int _starpu_hip_copy_data_from_cpu_to_hip(uintptr_t src, size_t src_offset, unsigned src_node, uintptr_t dst, size_t dst_offset, unsigned dst_node, size_t size, struct _starpu_async_channel *async_channel)
 {
 	int src_kind = starpu_node_get_kind(src_node);
 	int dst_kind = starpu_node_get_kind(dst_node);
@@ -1178,7 +1184,7 @@ int _starpu_hip_copy_data_from_cpu_to_hip(uintptr_t src, size_t src_offset, unsi
 					  hipMemcpyHostToDevice);
 }
 
-int _starpu_hip_copy2d_data_from_hip_to_cpu(uintptr_t src, size_t src_offset, unsigned src_node,
+static int _starpu_hip_copy2d_data_from_hip_to_cpu(uintptr_t src, size_t src_offset, unsigned src_node,
 					    uintptr_t dst, size_t dst_offset, unsigned dst_node,
 					    size_t blocksize, size_t numblocks, size_t ld_src, size_t ld_dst,
 					    struct _starpu_async_channel *async_channel)
@@ -1195,7 +1201,7 @@ int _starpu_hip_copy2d_data_from_hip_to_cpu(uintptr_t src, size_t src_offset, un
 					    hipMemcpyDeviceToHost);
 }
 
-int _starpu_hip_copy2d_data_from_hip_to_hip(uintptr_t src, size_t src_offset, unsigned src_node,
+static int _starpu_hip_copy2d_data_from_hip_to_hip(uintptr_t src, size_t src_offset, unsigned src_node,
 					    uintptr_t dst, size_t dst_offset, unsigned dst_node,
 					    size_t blocksize, size_t numblocks, size_t ld_src, size_t ld_dst,
 					    struct _starpu_async_channel *async_channel)
@@ -1215,7 +1221,7 @@ int _starpu_hip_copy2d_data_from_hip_to_hip(uintptr_t src, size_t src_offset, un
 					    hipMemcpyDeviceToDevice);
 }
 
-int _starpu_hip_copy2d_data_from_cpu_to_hip(uintptr_t src, size_t src_offset, unsigned src_node,
+static int _starpu_hip_copy2d_data_from_cpu_to_hip(uintptr_t src, size_t src_offset, unsigned src_node,
 					    uintptr_t dst, size_t dst_offset, unsigned dst_node,
 					    size_t blocksize, size_t numblocks, size_t ld_src, size_t ld_dst,
 					    struct _starpu_async_channel *async_channel)
@@ -1232,7 +1238,7 @@ int _starpu_hip_copy2d_data_from_cpu_to_hip(uintptr_t src, size_t src_offset, un
 					    hipMemcpyHostToDevice);
 }
 
-int _starpu_hip_is_direct_access_supported(unsigned node, unsigned handling_node)
+static int _starpu_hip_is_direct_access_supported(unsigned node, unsigned handling_node)
 {
 #if defined(STARPU_HAVE_HIP_MEMCPY_PEER)
 	(void) node;
