@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2020  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -33,27 +33,27 @@ starpu_data_handle_t A_handle, B_handle, C_handle, D_handle;
 static unsigned var = 0;
 starpu_data_handle_t var_handle;
 
-void f(void *descr[], void *arg)
+void func(void *descr[], void *arg)
 {
 	(void)descr;
 	(void)arg;
 
 	STARPU_SKIP_IF_VALGRIND;
 
-	usleep(200000);
+	starpu_usleep(200000);
 }
 
 static struct starpu_codelet cl_f =
 {
 	.modes = { STARPU_RW, STARPU_R, STARPU_RW },
-	.cpu_funcs = {f},
-	.cuda_funcs = {f},
-	.opencl_funcs = {f},
-	.cpu_funcs_name = {"f"},
+	.cpu_funcs = {func},
+	.cuda_funcs = {func},
+	.opencl_funcs = {func},
+	.cpu_funcs_name = {"func"},
 	.nbuffers = 3,
 };
 
-void g(void *descr[], void *arg)
+void g_cpu(void *descr[], void *arg)
 {
 	(void)descr;
 	(void)arg;
@@ -61,7 +61,7 @@ void g(void *descr[], void *arg)
 
 	unsigned *val = (unsigned *) STARPU_VARIABLE_GET_PTR(descr[0]);
 
-	usleep(100000);
+	starpu_usleep(100000);
 	*val = 42;
 }
 
@@ -74,7 +74,7 @@ void g_cuda(void *descr[], void *arg)
 	unsigned *val = (unsigned *) STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned value = 42;
 
-	usleep(100000);
+	starpu_usleep(100000);
 	cudaMemcpyAsync(val, &value, sizeof(value), cudaMemcpyHostToDevice, starpu_cuda_get_local_stream());
 	cudaStreamSynchronize(starpu_cuda_get_local_stream());
 }
@@ -89,11 +89,13 @@ void g_opencl(void *descr[], void *arg)
 	cl_mem val = (cl_mem) STARPU_VARIABLE_GET_PTR(descr[0]);
 	unsigned value = 42;
 
-	usleep(100000);
+	starpu_usleep(100000);
 	cl_command_queue queue;
 	starpu_opencl_get_current_queue(&queue);
 
-	clEnqueueWriteBuffer(queue, val, CL_TRUE, 0, sizeof(unsigned), (void *)&value, 0, NULL, NULL);
+	cl_int err;
+	err = clEnqueueWriteBuffer(queue, val, CL_TRUE, 0, sizeof(unsigned), (void *)&value, 0, NULL, NULL);
+	if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
 	clFinish(queue);
 }
 #endif
@@ -101,18 +103,18 @@ void g_opencl(void *descr[], void *arg)
 static struct starpu_codelet cl_g =
 {
 	.modes = { STARPU_RW, STARPU_R, STARPU_RW },
-	.cpu_funcs = {g},
+	.cpu_funcs = {g_cpu},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {g_cuda},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {g_opencl},
 #endif
-	.cpu_funcs_name = {"g"},
+	.cpu_funcs_name = {"g_cpu"},
 	.nbuffers = 3,
 };
 
-void h(void *descr[], void *arg)
+void h_cpu(void *descr[], void *arg)
 {
 	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
@@ -151,7 +153,9 @@ void h_opencl(void *descr[], void *arg)
 	cl_command_queue queue;
 	starpu_opencl_get_current_queue(&queue);
 
-	clEnqueueReadBuffer(queue, val, CL_TRUE, 0, sizeof(unsigned), (void *)&value, 0, NULL, NULL);
+	cl_int err;
+	err = clEnqueueReadBuffer(queue, val, CL_TRUE, 0, sizeof(unsigned), (void *)&value, 0, NULL, NULL);
+	if (STARPU_UNLIKELY(err != CL_SUCCESS)) STARPU_OPENCL_REPORT_ERROR(err);
 	clFinish(queue);
 
 	FPRINTF(stderr, "VAR %u (should be 42)\n", value);
@@ -162,14 +166,14 @@ void h_opencl(void *descr[], void *arg)
 static struct starpu_codelet cl_h =
 {
 	.modes = { STARPU_RW, STARPU_R, STARPU_RW },
-	.cpu_funcs = {h},
+	.cpu_funcs = {h_cpu},
 #ifdef STARPU_USE_CUDA
 	.cuda_funcs = {h_cuda},
 #endif
 #ifdef STARPU_USE_OPENCL
 	.opencl_funcs = {h_opencl},
 #endif
-	.cpu_funcs_name = {"h"},
+	.cpu_funcs_name = {"h_cpu"},
 	.nbuffers = 3
 };
 
@@ -181,10 +185,10 @@ int main(int argc, char **argv)
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
-	A = (unsigned *) malloc(VECTORSIZE*sizeof(unsigned));
-	B = (unsigned *) malloc(VECTORSIZE*sizeof(unsigned));
-	C = (unsigned *) malloc(VECTORSIZE*sizeof(unsigned));
-	D = (unsigned *) malloc(VECTORSIZE*sizeof(unsigned));
+	starpu_malloc((void **)&A, VECTORSIZE*sizeof(unsigned));
+	starpu_malloc((void **)&B, VECTORSIZE*sizeof(unsigned));
+	starpu_malloc((void **)&C, VECTORSIZE*sizeof(unsigned));
+	starpu_malloc((void **)&D, VECTORSIZE*sizeof(unsigned));
 
 	starpu_vector_data_register(&A_handle, STARPU_MAIN_RAM, (uintptr_t)A, VECTORSIZE, sizeof(unsigned));
 	starpu_vector_data_register(&B_handle, STARPU_MAIN_RAM, (uintptr_t)B, VECTORSIZE, sizeof(unsigned));
@@ -200,7 +204,7 @@ int main(int argc, char **argv)
 	starpu_data_set_sequential_consistency_flag(D_handle, 0);
 #endif
 
-	/* 	f(Ar, Brw): sleep 
+	/* 	f(Ar, Brw): sleep
 	 *	g(Br; Crw); sleep, var = 42
 	 *	h(Cr; Drw); check that var == 42
 	 */
@@ -241,10 +245,10 @@ int main(int argc, char **argv)
 
 	starpu_data_unregister(var_handle);
 
-	free(A);
-	free(B);
-	free(C);
-	free(D);
+	starpu_free_noflag(A, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(B, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(C, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(D, VECTORSIZE*sizeof(unsigned));
 
 	starpu_shutdown();
 
@@ -258,10 +262,10 @@ enodev:
 
 	starpu_data_unregister(var_handle);
 
-	free(A);
-	free(B);
-	free(C);
-	free(D);
+	starpu_free_noflag(A, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(B, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(C, VECTORSIZE*sizeof(unsigned));
+	starpu_free_noflag(D, VECTORSIZE*sizeof(unsigned));
 	fprintf(stderr, "WARNING: No one can execute this task\n");
 	/* yes, we do not perform the computation but we did detect that no one
  	 * could perform the kernel, so this is not an error from StarPU */
