@@ -1,8 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2012                                Inria
- * Copyright (C) 2011,2013-2014,2016                      Université de Bordeaux
- * Copyright (C) 2010,2012,2015-2017                      CNRS
+ * Copyright (C) 2010-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -25,8 +23,9 @@
 
 extern struct starpu_opencl_program opencl_code;
 
-void opencl_codelet_incA(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void opencl_codelet_incA(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_mem val = (cl_mem)STARPU_VECTOR_GET_DEV_HANDLE(descr[0]);
@@ -52,8 +51,9 @@ void opencl_codelet_incA(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 	starpu_opencl_release_kernel(kernel);
 }
 
-void opencl_codelet_incC(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
+void opencl_codelet_incC(void *descr[], void *arg)
 {
+	(void)arg;
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_mem val = (cl_mem)STARPU_VECTOR_GET_DEV_HANDLE(descr[0]);
