@@ -1,6 +1,6 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011-2021  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+ * Copyright (C) 2011-2025  University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -41,7 +41,7 @@ void func_cpu(void *descr[], void *_args)
 struct starpu_codelet codelet =
 {
 	.cpu_funcs = {func_cpu},
-	/* starpu_task_get_current() doesn't work on MIC */
+	/* starpu_task_get_current() doesn't work on MPI Master Slave */
 	/* .cpu_funcs_name = {"func_cpu"}, */
 	.nbuffers = STARPU_VARIABLE_NBUFFERS,
 };
@@ -49,7 +49,7 @@ struct starpu_codelet codelet =
 struct starpu_codelet codelet_minus1 =
 {
 	.cpu_funcs = {func_cpu},
-	/* starpu_task_get_current() doesn't work on MIC */
+	/* starpu_task_get_current() doesn't work on MPI Master Slave */
 	/* .cpu_funcs_name = {"func_cpu"}, */
 	.nbuffers = STARPU_NMAXBUFS-1,
 };
@@ -57,7 +57,7 @@ struct starpu_codelet codelet_minus1 =
 struct starpu_codelet codelet_exactly =
 {
 	.cpu_funcs = {func_cpu},
-	/* starpu_task_get_current() doesn't work on MIC */
+	/* starpu_task_get_current() doesn't work on MPI Master Slave */
 	/* .cpu_funcs_name = {"func_cpu"}, */
 	.nbuffers = STARPU_NMAXBUFS,
 };
@@ -65,7 +65,7 @@ struct starpu_codelet codelet_exactly =
 struct starpu_codelet codelet_plus1 =
 {
 	.cpu_funcs = {func_cpu},
-	/* starpu_task_get_current() doesn't work on MIC */
+	/* starpu_task_get_current() doesn't work on MPI Master Slave */
 	/* .cpu_funcs_name = {"func_cpu"}, */
 	.nbuffers = STARPU_NMAXBUFS+1,
 };
@@ -73,7 +73,7 @@ struct starpu_codelet codelet_plus1 =
 struct starpu_codelet codelet_plus5 =
 {
 	.cpu_funcs = {func_cpu},
-	/* starpu_task_get_current() doesn't work on MIC */
+	/* starpu_task_get_current() doesn't work on MPI Master Slave */
 	/* .cpu_funcs_name = {"func_cpu"}, */
 	.nbuffers = STARPU_NMAXBUFS+5,
 };
@@ -135,6 +135,13 @@ int main(void)
 {
         int *x;
         int i, ret, loop;
+	struct starpu_conf conf;
+
+	starpu_conf_init(&conf);
+	starpu_conf_noworker(&conf);
+	conf.ncpus = -1;
+	conf.nmpi_ms = -1;
+	conf.ntcpip_ms = -1;
 
 #ifdef STARPU_QUICK_CHECK
 	int nloops = 4;
@@ -142,7 +149,7 @@ int main(void)
 	int nloops = 16;
 #endif
 
-	ret = starpu_init(NULL);
+	ret = starpu_init(&conf);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
@@ -179,7 +186,7 @@ int main(void)
 		if (ret == -ENODEV) goto enodev;
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 
-		/* Test datas one after the other, but less than NMAXBUFS */
+		/* Test data one after the other, but less than NMAXBUFS */
 		expected[0]++;
 		for (i = 1; i < STARPU_NMAXBUFS-1 && i < 10; i++)
 			expected[i]++;
@@ -246,10 +253,7 @@ int main(void)
 		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_insert");
 #endif
 
-
-
-
-		/* Test datas one after the other, but more than NMAXBUFS */
+		/* Test data one after the other, but more than NMAXBUFS */
 		for (i = 0; i < STARPU_NMAXBUFS+5 && i < 10; i++)
 			expected[i]++;
 		ret = starpu_task_insert(&codelet,
