@@ -1,6 +1,6 @@
 # StarPU --- Runtime system for heterogeneous multicore architectures.
 #
-# Copyright (C) 2016-2022  Université de Bordeaux, CNRS (LaBRI UMR 5800), Inria
+# Copyright (C) 2016-2025   University of Bordeaux, CNRS (LaBRI UMR 5800), Inria
 #
 # StarPU is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -38,11 +38,12 @@ run()
 	sched=$1
 
 	set +e
-	STARPU_SCHED=$sched $STARPU_SUB_PARALLEL $STARPU_LAUNCH $(dirname $0)/$TEST "$@"
+	STARPU_SCHED=$sched $STARPU_SUB_PARALLEL $MS_LAUNCHER $STARPU_LAUNCH $(dirname $0)/$TEST "$@"
 	ret=$?
 	set -e
 	if test $ret = 0
 	then
+		( echo PASS: STARPU_SCHED=$sched ./microbenchs/$TEST >&9 ) 2> /dev/null || true
 		echo "SUCCESS: STARPU_SCHED=$sched ./microbenchs/$TEST"
 		return 0
 	fi
